/**
 * Pinpoint Node.js Agent
 * Copyright 2020-present NAVER Corp.
 * Apache License v2.0
 */

'use strict'

const semver = require('semver')
const log = require('../../utils/logger')
const InstrumentMethod = require('../instrument-method')
const IoredisSendCommandInterceptor = require('./ioredis/ioredis-send-command-interceptor')

module.exports = function (agent, version, ioredis) {
  if (!semver.satisfies(version, '>=2.0.0 <5.0.0')) {
    log.debug('ioredis version %s not supported - aborting...', version)
    return ioredis
  }

  InstrumentMethod.make(ioredis.prototype, 'sendCommand').addScopedInterceptor(new IoredisSendCommandInterceptor())  
  return ioredis
}
